import 'dart:convert';
import 'dart:typed_data';

import 'package:test/test.dart';

import 'package:dart_git/utils/kvlm.dart';

void main() {
  var rawStr = '''tree 29ff16c9c14e2652b22f8b78bb08a5a07930c147
parent 206941306e8a8af65b66eaaaea388a7ae24d49a0
parent 206941306e8a8af65b66eaaaea388a7ae24d49a2
author Thibault Polge <thibault@thb.lt> 1527025023 +0200
committer Thibault Polge <thibault@thb.lt> 1527025044 +0200
gpgsig -----BEGIN PGP SIGNATURE-----
 iQIzBAABCAAdFiEExwXquOM8bWb4Q2zVGxM2FxoLkGQFAlsEjZQACgkQGxM2FxoL
 kGQdcBAAqPP+ln4nGDd2gETXjvOpOxLzIMEw4A9gU6CzWzm+oB8mEIKyaH0UFIPh
 rNUZ1j7/ZGFNeBDtT55LPdPIQw4KKlcf6kC8MPWP3qSu3xHqx12C5zyai2duFZUU
 wqOt9iCFCscFQYqKs3xsHI+ncQb+PGjVZA8+jPw7nrPIkeSXQV2aZb1E68wa2YIL
 3eYgTUKz34cB6tAq9YwHnZpyPx8UJCZGkshpJmgtZ3mCbtQaO17LoihnqPn4UOMr
 V75R/7FjSuPLS8NaZF4wfi52btXMSxO/u7GuoJkzJscP3p4qtwe6Rl9dc1XC8P7k
 NIbGZ5Yg5cEPcfmhgXFOhQZkD0yxcJqBUcoFpnp2vu5XJl2E5I/quIyVxUXi6O6c
 /obspcvace4wy8uO0bdVhc4nJ+Rla4InVSJaUaBeiHTW8kReSFYyMmDCzLjGIu1q
 doU61OM3Zv1ptsLu3gUE6GU27iWYj2RWN3e3HE4Sbd89IFwLXNdSuM0ifDLZk7AQ
 WBhRhipCCgZhkj9g2NEk7jRVslti1NdN5zoQLaJNqSwO1MtxTmJ15Ksk3QP6kfLB
 Q52UWybBzpaP9HEd4XnR+HuQ4k2K0ns2KgNImsNvIyFwbpMUyUWLMPimaV1DWUXo
 5SBjDB/V/W2JBFR+XKHFJeFwYhj7DD/ocsGr4ZMx/lgc8rjIBkI=
 =lgTX
 -----END PGP SIGNATURE-----

Create first draft''';

  test('Parses KVLM Properly', () async {
    var raw = utf8.encode(rawStr);
    var map = kvlmParse(raw as Uint8List);

    expect(map.length, equals(6));

    expect(map['tree'], equals('29ff16c9c14e2652b22f8b78bb08a5a07930c147'));

    var p = map['parent'];
    expect(p[0] as String?, equals('206941306e8a8af65b66eaaaea388a7ae24d49a0'));
    expect(p[1] as String?, equals('206941306e8a8af65b66eaaaea388a7ae24d49a2'));

    expect(map['gpgsig'], equals('''-----BEGIN PGP SIGNATURE-----
iQIzBAABCAAdFiEExwXquOM8bWb4Q2zVGxM2FxoLkGQFAlsEjZQACgkQGxM2FxoL
kGQdcBAAqPP+ln4nGDd2gETXjvOpOxLzIMEw4A9gU6CzWzm+oB8mEIKyaH0UFIPh
rNUZ1j7/ZGFNeBDtT55LPdPIQw4KKlcf6kC8MPWP3qSu3xHqx12C5zyai2duFZUU
wqOt9iCFCscFQYqKs3xsHI+ncQb+PGjVZA8+jPw7nrPIkeSXQV2aZb1E68wa2YIL
3eYgTUKz34cB6tAq9YwHnZpyPx8UJCZGkshpJmgtZ3mCbtQaO17LoihnqPn4UOMr
V75R/7FjSuPLS8NaZF4wfi52btXMSxO/u7GuoJkzJscP3p4qtwe6Rl9dc1XC8P7k
NIbGZ5Yg5cEPcfmhgXFOhQZkD0yxcJqBUcoFpnp2vu5XJl2E5I/quIyVxUXi6O6c
/obspcvace4wy8uO0bdVhc4nJ+Rla4InVSJaUaBeiHTW8kReSFYyMmDCzLjGIu1q
doU61OM3Zv1ptsLu3gUE6GU27iWYj2RWN3e3HE4Sbd89IFwLXNdSuM0ifDLZk7AQ
WBhRhipCCgZhkj9g2NEk7jRVslti1NdN5zoQLaJNqSwO1MtxTmJ15Ksk3QP6kfLB
Q52UWybBzpaP9HEd4XnR+HuQ4k2K0ns2KgNImsNvIyFwbpMUyUWLMPimaV1DWUXo
5SBjDB/V/W2JBFR+XKHFJeFwYhj7DD/ocsGr4ZMx/lgc8rjIBkI=
=lgTX
-----END PGP SIGNATURE-----'''));

    expect(map['_'], equals('Create first draft'));
    expect(utf8.decode(kvlmSerialize(map)), equals(rawStr));
  });

  test('Parse test commit', () {
    var raw = utf8.encode('''tree bd11e76b46ad288c6e80e4db357dadf676906897
author Vishesh Handa <me@vhanda.in> 1601767396 +0200
committer Vishesh Handa <me@vhanda.in> 1601767396 +0200

SDf
''');

    var map = kvlmParse(raw as Uint8List);
    expect(map.length, equals(4));
    expect(map['tree'], equals('bd11e76b46ad288c6e80e4db357dadf676906897'));
    expect(map['author'], 'Vishesh Handa <me@vhanda.in> 1601767396 +0200');
    expect(map['committer'], 'Vishesh Handa <me@vhanda.in> 1601767396 +0200');
    expect(map['_'], 'SDf\n');
  });

  test('Parse empty', () {
    var raw = Uint8List(0);
    var map = kvlmParse(raw);
    expect(map.length, equals(0));
  });
}
